#!/bin/sh
# \
exec fltkwish "$0" -- ${1+"$@"}
#
# --- change.tcl --- Change log manager
#
# Copyright(C) I.B.Findleton, 2002. All Rights Reserved
#
# This script takes input from the user and writes formatted records into
# a file called "changes.log". It is a simple method of keeping track of
# modifications to the code. It will also display the history of logged
# changes for a source module.
#

Scheme plastic

#if { $tcl_platform(platform) == "windows" } {
#	Scheme image -f $Fltk(Library)/images/clouds.jpg
#} else {
#	Scheme image -f /source/Fltk-0.3/scripts/images/clouds.jpg
#	}

#

# Get the version of the source code

proc GetVersion { name } {

	if { [catch { open $name r } fd ] } {
		puts stderr "Can't find the version file $name"
	} else {
		set list [split [read $fd] "\n\r \t"]
		set ord [lsearch $list "VERSION"]
		if { $ord != -1 } {
			return [lindex $list [incr ord 2]]
		} else {
			puts stderr "Can't find version of the source code!"
			}
		close $fd
		}
		
	return "0.4"
	}
	
# Get the patch level of the source code

proc GetPatchLevel { name } {

	if { [catch { open $name r } fd ] } {
		puts stderr "Can't find the version file $name"
	} else {
		set list [split [read $fd] "\n\r \t"]
		set ord [lsearch $list "PATCHLEVEL"]
		if { $ord != -1 } {
			return [lindex $list [incr ord 2]]
		} else {
			puts stderr "Can't find patch level of the source code!"
			}
		close $fd
		}
		
	return "0.1"
	}

set Data(FileName) changes.log
set Data(Module) ""
set Data(Comment) ""
set Data(VersionFile) ../version
set Data(Version) [GetVersion $Data(VersionFile)]
set Data(PatchLevel) [GetPatchLevel $Data(VersionFile)]
set Data(ValidModule) 0
set Data(Status) ""

# Update the log file

proc UpdateLog { } {

	global Data
	
	set fd [open $Data(FileName) a+]
	
	if { $fd != "" } {
		puts $fd "[clock format [clock seconds]]*$Data(Version).$Data(PatchLevel)*$Data(Module)*$Data(Comment)"
		close $fd
		}

	set Data(Comment) ""

	LoadHistory	
	}
	
# Load the history for a module

proc LoadHistory { } {

	global Data
	
	set fd [open $Data(FileName) r]
	
	if { $fd != "" } {
		$Data(History) clear
		set list [split [read $fd] \n]
		close $fd
		while { [set idx [lsearch -glob $list *$Data(Module)*]] != -1 } {
			set newlist [lrange $list [expr $idx + 1] end]
			set next [lsearch $newlist *]
			if { $next == -1 } { set next [llength $list] } else { incr next }
			while { $idx < $next } {
				$Data(History) add [lindex $list $idx]
				incr idx
				}
			if { $idx < [llength $list] } {
				set list [lrange $list $idx end]
				}
			}
		}
	}

# Handle a return key from the input widget

proc HandleKey { w name } {

	global Data

	if { $name == "Enter" } {
		set Data(Status) ""
		if { [file exists $Data(Module)] } {
			set Data(Comment) ""
			LoadHistory
			set Data(ValidModule) 1
		} else {
			set Data(Comment) ""
			set Data(Status) "File $Data(Module) does not exist!"
			set Data(ValidModule) 0
			}
		}
	}

proc HandleComment { w name } {

	global Data

	if { $name == "Enter" } {
		UpdateLog
		} 
	}

# Handle a change in the version information

proc HandleVersionChange { w name } {

	global Data

	if { $name == "Enter" } {
		set fd [open $Data(FileName) a+]
		if { $fd != "" } {
			puts $fd "[clock format [clock seconds]]*Version change to $Data(Version).$Data(PatchLevel)"
			close $fd
			}
		}
	}
	
# Edit the current module

proc Edit {} {

	global Data
	
	eval exec nedit $Data(Module) &
	}

# Build the GUI to handle the operator input
			
set f [Package t.t -w 600 -h 200 -orientation vertical]

# Inputs for the module, version and the patch level values

set f0 [Package $f.module -orientation horizontal]
Label $f0.l1 -text "Module Name" -align left,inside -relief flat
Input $f0.i1 -variable Data(Module) -w 200 \
			-tooltip "Enter a valid source module name" 
Label $f0.l2 -text "Version" -align left,inside -relief flat
Input $f0.i2 -variable Data(Version) -w 60 \
			-tooltip "Enter the current version number"
Label $f0.l3 -text "Patch Level" -align left,inside -relief flat
Input $f0.i3 -variable Data(PatchLevel) -w 60 \
			-tooltip "Enter the current patch level"

# When someone presses the enter key, we want to take action

Bind $f0.i1 <KeyPress> { HandleKey %W %K }
Bind $f0.i2 <KeyPress> { HandleVersionChange %W %K }
Bind $f0.i3 <KeyPress> { HandleVersionChange %W %K }

# This is the widget that reads the comment about the change

Input $f.comment -variable Data(Comment) -width 600 -height 20 \
		-statevariable Data(ValidModule) \
		-tooltip "Enter a comment describing the changes made"

Bind $f.comment <KeyPress> { HandleComment %W %K } 

# This listbox will hold the current history for a module

set Data(History) [Listbox $f.history -width 600 -height 100]

# Somewhere to put error messages

Label $f.status -align left,inside -variable Data(Status) \
		-tooltip "Displays error messages"

# A button to kill the application

set f2 [Package $f.actions -orientation horizontal]

Button $f2.edit -text Edit -command Edit \
		-tooltip "Edit the current module file"
Button $f2.quit -text "Dismiss" -command Exit \
		-tooltip "Terminate the application"

# Display the GUI

Show t

Wm title t "Change Log Manager V1.1"

	
